/*
** flite.c - Interface to the CMU Flite tts system from voxd VoiceOver daemon
** Copyright (C) 2017  Lew Pitcher
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License along
** with this program; if not, write to the Free Software Foundation, Inc.,
** 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**
** Lew Pitcher <lew.pitcher@digitalfreehold.ca>
*/

/*
** NOTE: This version REQUIRES libraries from flite v2.0.0
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <flite/flite.h>

cst_voice *register_cmu_us_rms(char *);	/* libflite_cmu_us_rms */
void usenglish_init(cst_voice *v);	/* libflite_usenglish */
cst_lexicon *cmulex_init(void);		/* libflite_cmulex */

void InitFlite(void)
{
  flite_init();

  flite_add_lang("eng",usenglish_init,cmulex_init);
  flite_add_lang("usenglish",usenglish_init,cmulex_init);

  flite_voice_list = cons_val(voice_val(register_cmu_us_rms(NULL)),flite_voice_list);
  flite_voice_list = val_reverse(flite_voice_list);
}

cst_voice *GetVoice(char *path, float tempo, float pitch, float intonation)
{
  cst_voice *Voice = NULL;

  if ((Voice = flite_voice_select(path)))
  {
    if (tempo > 0.0)
      feat_set_float(Voice->features,"duration_stretch",tempo);
    if (pitch > 0.0)
      feat_set_float(Voice->features,"int_f0_target_mean",pitch);
    if (intonation > 0.0)
      feat_set_float(Voice->features,"int_f0_target_stddev",intonation);
  }
  return Voice;
}

void Say(cst_voice *Voice, char *format, ...)
{
  va_list	ap, 	/* our va_list */
		pp; 	/* vsnprintf va_list */

  size_t	textsize;
  char		dummy[1],
		*text = NULL;

  va_start(ap,format);

  va_copy(pp,ap);
  textsize = 2 + vsnprintf(dummy,sizeof(dummy),format,pp);

  if ((text = malloc(textsize)))
  {
    va_copy(pp,ap);
    vsnprintf(text,textsize,format,pp);
    flite_text_to_speech(text,Voice,"play");
    free(text);
  }
  va_end(ap);
}