%{
/* voxd.lex - Lexer for voxd VoiceOver daemon
** Copyright (C) 2017  Lew Pitcher
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License along
** with this program; if not, write to the Free Software Foundation, Inc.,
** 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**
** Lew Pitcher <lew.pitcher@digitalfreehold.ca>
*/

/* lex globals, constants and macros */
#include <stdlib.h>
#include <string.h>
#include "y.tab.h"

extern unsigned int LineNumber;

%}
%%
[Pp]rologue		{ return PROLOGUE;	}
[Ee]pilogue		{ return EPILOGUE;	}
[Tt]empo		{ return TEMPO;		}
[Pp]itch		{ return PITCH;		}
[Ii]ntonation|[Tt]one	{ return INTONATION;	}
(is\ )?voiced\ by	{ return VOICEDBY;	}
has\ voice		{ return HASVOICE;	}
\(			{ return LPAREN;	}
\)			{ return RPAREN;	}
=			{ return EQUALS;	}
,			{ return COMMA;		}
\"[^\n"]*\"		{ /* drop the leading and trailing quote */
			  *(yytext + strlen(yytext) - 1) = 0;		/* kill off closing quote */
			  yylval.String = strdup(yytext+1);	/* copy without opening quote */
			  return QUOTEDSTRING;
			}
[A-Za-z][-_.A-Za-z]*[A-Za-z]	{ yylval.String = strdup(yytext); return NAME;	}
[0-9]+\.[0-9]+		{ yylval.Float  = strtof(yytext,NULL); return FLOATPT; }

[#;][^\n]*		/* Ignore comments */ ;
\n			{ ++LineNumber; }
[ \t]+			/* Ignore Whitespace */ ;
%%
