/* voxreadline - commandline tool to invoke speech from voxd VoiceOver daemon
** Copyright (C) 2017  Lew Pitcher
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License along
** with this program; if not, write to the Free Software Foundation, Inc.,
** 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
**
** Lew Pitcher <lew.pitcher@digitalfreehold.ca>
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#include "voxconfig.h"

int main(int argc, char *argv[])
{
  int rc = EXIT_FAILURE;
  int	opt;
  extern char *optarg;
  char	*SocketPath = NULL;
  int	argerrors = 0;
  char  *role,
	*line;

  while ((opt = getopt(argc,argv,"s:")) != -1)
  {
    switch (opt)
    {
      case 's':	/* change socket path */
	free(SocketPath);
	if ((SocketPath = realpath(optarg,NULL)) == NULL)
	  argerrors = 1;
	break;

      default:	/* anything else */
	argerrors = 1;
	break;
    }
  }
  if (SocketPath == NULL) SocketPath = strdup(SOCKETPATH);

  switch (argc - optind)
  {
    case 2:
      role = argv[optind];
      line = argv[optind+1];
      break;

    case 1:
      role = argv[optind];
      {
	size_t linelen;

	linelen = MAXMSGLEN - strlen(role) - 2;

	if ((line = malloc(linelen)))
	{
	  ssize_t maxbytes;

	  maxbytes = read(STDIN_FILENO,line,linelen);
	  line[maxbytes] = 0;
	}
      }
      break;

    default:
      argerrors = 1;
      break;
  }

  if (argerrors == 0)
  {
    int script;

    if ((script = socket(AF_UNIX,SOCK_DGRAM,0)) >= 0)
    {
      struct sockaddr_un serveraddress;
      char message[MAXMSGLEN+1];
      int msglen;

      memset(&serveraddress,0,sizeof serveraddress);
      serveraddress.sun_family = AF_LOCAL;
      strcpy(serveraddress.sun_path,SocketPath);

      msglen = snprintf(message, sizeof message, "%s: %s",role,line);
      if (msglen+1 < sizeof message)
      {
	if (sendto(script,message,(sizeof message)-1,0,(struct sockaddr *)&serveraddress,sizeof serveraddress) < 0)
	{
	  fprintf(stderr,"%s: Cant send lines via %s\n",argv[0],SocketPath);
	  perror("sendto()");
	}
	else rc = EXIT_SUCCESS;
      }
      else fprintf(stderr,"Message size %d exceeds buffer size %d\n",msglen, sizeof message);
    }
    else fprintf(stderr,"Cant create AF_UNIX SOCK_DGRAM socket\n");
  }
  else fprintf(stderr,"Usage: %s [-s socketpath] role [line]\n",argv[0]);

  return rc;
}
